package de.unisaar.jfap.ex7;

import java.util.Comparator;

/**
 * Interface of a functional object used to compare two instances of the argument type.
 *
 * @author Bernd Kiefer, DFKI
 * @author Jörg Steffen, DFKI
 * @param <T> argument type this predicates works on
 */
public interface BinaryPredicate<T> {

  /**
   * Compare the given objects of type ArgumentTyp and returns {@code true} if they are compatible.
   *
   * @param arg1 the first argument
   * @param arg2 the second argument
   * @return a flag indicating if the arguments are compatible
   */
  boolean compare(T arg1, T arg2);

  /**
   * Functional object used to check if two instances of the argument type are equal.
   *
   * @param <T> the type of arguments to compare
   */
  class EqualsPredicate<T> implements BinaryPredicate<T> {

    @Override
    public boolean compare(T arg1, T arg2) {
      return arg1.equals(arg2);
    }
  }

  /**
   * Functional object used to compare two instances of the argument type using their internal
   * comparison method.
   *
   * @param <T> the type of arguments to compare
   */
  class ComparableComparator<T extends Comparable<T>> implements Comparator<T> {

    @Override
    public int compare(T arg1, T arg2) {
      return arg1.compareTo(arg2);
    }
  }
}
