package de.unisaar.jfap.ex7;

import java.util.Map;

/**
 * Edge in the directed graph.
 *
 * @author Bernd Kiefer, DFKI
 * @author Jörg Steffen, DFKI
 * @param <E> the type of info assigned to this edge
 */
public class Edge<E> {

  private E info;
  private int from;
  private int to;

  /**
   * Create a new edge and adds it to the appropriate vertex's edge lists. This constructor has
   * default visibility because only {@link DirectedGraph#newEdge(Object, int, int)} should be
   * able to create a new edge instance.
   *
   * @param info the info assigned to this edge
   */
  Edge(E info) {

    this.info = info;
  }

  public E getInfo() {
    return this.info;
  }

  public void setInfo(E info) {
    this.info = info;
  }

  public int getStartVertex() {
    return this.from;
  }

  void setStartVertex(int startVertex) {
    this.from = startVertex;
  }

  public int getEndVertex() {

    return this.to;
  }

  void setEndVertex(int endVertex) {
    this.to = endVertex;
  }

  @Override
  public String toString() {
    return String.format("%d--%s-->%d", this.from, this.info, this.to);
  }

  /**
   * Return a string representation using the given vertex names.
   *
   * @param vertexNames a property map with the vertex names
   * @return the string representation
   */
  public String toString(Map<Integer, ?> vertexNames) {
    if (vertexNames == null) {
      return toString();
    }
    return String.format(
        "%s--%s-->%s", vertexNames.get(this.from), this.info, vertexNames.get(this.to));
  }
}
