package de.unisaar.jfap.ex7.visitors;

import de.unisaar.jfap.ex7.DirectedGraph;
import de.unisaar.jfap.ex7.Edge;

/**
 * Extend the {@link GraphVisitor} interface with methods required only in depth first search.
 *
 * @author Jörg Steffen, DFKI
 * @param <E> the type of information assigned to the edges of the traversed graph
 * @param <V> the type of information assigned to the vertices of the traversed graph
 */
public interface DfsGraphVisitor<E, V> extends GraphVisitor<E, V> {

  /**
   * Call when a tree edge is used, before the DFS recursive call.
   *
   * @param e the tree edge
   * @param g the graph to which the tree edge belongs to
   */
  void treeEdgeBefore(Edge<E> e, DirectedGraph<E, V> g);

  /**
   * Call when a tree edge is used, after the DFS recursive call.
   *
   * @param e the tree edge
   * @param g the graph to which the tree edge belongs to
   */
  void treeEdgeAfter(Edge<E> e, DirectedGraph<E, V> g);
}
