package de.unisaar.jfap.ex7.visitors;

import de.unisaar.jfap.ex7.DirectedGraph;
import de.unisaar.jfap.ex7.Edge;

/**
 * Define the common interface for visitors used in both depth first search and breadth first
 * search. Depending on the kind of search to be used with the visitor, use the interfaces {@link
 * DfsGraphVisitor} and {@link BfsGraphVisitor}.
 *
 * @author Bernd Kiefer, DFKI
 * @author Jörg Steffen, DFKI
 * @param <E> the type of information assigned to the edges of the traversed graph
 * @param <V> the type of information assigned to the vertices of the traversed graph
 */
public interface GraphVisitor<E, V> {

  /**
   * In the BFS/DFS variants that visits each vertex of the graph, this method is called for the
   * start vertices where the real search begins.
   *
   * @param v the start vertex
   * @param g the graph to which the start vertex belongs to
   */
  void startVertex(int v, DirectedGraph<E, V> g);

  /**
   * Call when a vertex turns from white to gray.
   *
   * @param v the vertex that is discovered
   * @param g the graph to which the discovered vertex belongs to
   */
  void discoverVertex(int v, DirectedGraph<E, V> g);

  /**
   * Call when a vertex turns from gray to black.
   *
   * @param v the vertex that is left
   * @param g the graph to which the left vertex belongs to
   */
  void finishVertex(int v, DirectedGraph<E, V> g);

  /**
   * Call when a non-tree edge is used.
   *
   * @param e the non-tree edge
   * @param g the graph to which the non-tree edge belongs to
   */
  void nonTreeEdge(Edge<E> e, DirectedGraph<E, V> g);
}
