package de.unisaar.jfap.ex7;

import static org.assertj.core.api.Assertions.assertThat;

import de.unisaar.jfap.ex7.BinaryPredicate.EqualsPredicate;
import java.io.IOException;
import java.util.Map;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

class CollectTimesVisitorTest {

  DirectedGraph<String, String> graph;

  @BeforeEach
  void init() throws IOException {
    String graphString =
        String.format(
            "A --> B E%n"
                + "B --> C E%n"
                + "C --> D%n"
                + "D --> B%n"
                + "E --> D F%n"
                + "F --> D%n"
                + "G --> H I%n"
                + "H --> A E%n"
                + "I --> G H%n");
    graph = DirectedGraph.readGraph(graphString);
  }

  @Test
  void testTimesVisitor() {
    CollectTimesVisitor<String, String> timesVisitor = new CollectTimesVisitor<>();
    graph.dfs(timesVisitor);
    BinaryPredicate<String> pred = new EqualsPredicate<>();
    Map<Integer, Integer> expectedDiscoveryTimes =
        Map.of(
            graph.findVertex("A", pred), 1,
            graph.findVertex("B", pred), 2,
            graph.findVertex("C", pred), 3,
            graph.findVertex("D", pred), 4,
            graph.findVertex("E", pred), 7,
            graph.findVertex("F", pred), 8,
            graph.findVertex("G", pred), 13,
            graph.findVertex("H", pred), 14,
            graph.findVertex("I", pred), 16);
    assertThat(timesVisitor.getDiscoveryMap())
        .containsExactlyInAnyOrderEntriesOf(expectedDiscoveryTimes);

    Map<Integer, Integer> expectedFinishTimes =
        Map.of(
            graph.findVertex("A", pred), 12,
            graph.findVertex("B", pred), 11,
            graph.findVertex("C", pred), 6,
            graph.findVertex("D", pred), 5,
            graph.findVertex("E", pred), 10,
            graph.findVertex("F", pred), 9,
            graph.findVertex("G", pred), 18,
            graph.findVertex("H", pred), 15,
            graph.findVertex("I", pred), 17);
    assertThat(timesVisitor.getFinishMap()).containsExactlyInAnyOrderEntriesOf(expectedFinishTimes);

    assertThat(timesVisitor.getTreeEdges())
        .extracting(Edge::getInfo)
        .containsExactlyInAnyOrder("AB", "BC", "CD", "BE", "EF", "GH", "GI");
    assertThat(timesVisitor.getForwardEdges())
        .extracting(Edge::getInfo)
        .containsExactlyInAnyOrder("AE");
    assertThat(timesVisitor.getBackEdges())
        .extracting(Edge::getInfo)
        .containsExactlyInAnyOrder("DB", "IG");
    assertThat(timesVisitor.getCrossEdges())
        .extracting(Edge::getInfo)
        .containsExactlyInAnyOrder("ED", "FD", "HA", "HE", "IH");
  }
}
