package de.unisaar.jfap.ex7;

import static org.assertj.core.api.Assertions.assertThat;

import java.util.List;
import org.junit.jupiter.api.Test;

class DirectedGraphTest {

  @Test
  void testReader() {
    String graphString =
        String.format(
            "A --> B E%n"
                + "B --> C E%n"
                + "C --> D%n"
                + "D --> B%n"
                + "E --> D F%n"
                + "F --> D%n"
                + "G --> H I%n"
                + "H --> A E%n"
                + "I --> G H%n");
    DirectedGraph<String, String> graph = DirectedGraph.readGraph(graphString);
    List<String> expected =
        List.of(
            "A--AB-->B",
            "A--AE-->E",
            "B--BC-->C",
            "B--BE-->E",
            "E--ED-->D",
            "E--EF-->F",
            "C--CD-->D",
            "D--DB-->B",
            "F--FD-->D",
            "G--GH-->H",
            "G--GI-->I",
            "H--HA-->A",
            "H--HE-->E",
            "I--IG-->G",
            "I--IH-->H");
    assertThat(graph.toString().lines().toList()).containsExactlyInAnyOrderElementsOf(expected);
  }

  @Test
  void testReaderWithWeight() {
    String graphString =
        String.format(
            "s --> w(2) z(4)%n"
                + "z --> w(5) y(9)%n"
                + "v --> w(1) s(6)%n"
                + "w --> x(6) q(3)%n"
                + "t --> u(1) v(9) s(8)%n"
                + "u --> t(5) v(7)%n"
                + "x --> z(8) u(7)%n"
                + "q --> x(2)%n"
                + "y --> x(5)%n"
                + "r --> s(1)");
    DirectedGraph<Integer, String> graph = DirectedGraph.readGraphWithWeights(graphString);
    List<String> expected =
        List.of(
            "s--2-->w",
            "s--4-->z",
            "w--6-->x",
            "w--3-->q",
            "z--5-->w",
            "z--9-->y",
            "y--5-->x",
            "v--1-->w",
            "v--6-->s",
            "x--8-->z",
            "x--7-->u",
            "q--2-->x",
            "t--1-->u",
            "t--9-->v",
            "t--8-->s",
            "u--5-->t",
            "u--7-->v",
            "r--1-->s");
    assertThat(graph.toString().lines().toList()).containsExactlyInAnyOrderElementsOf(expected);
  }
}
